/* Gateway routine for calling C++ from Matlab */

#include <mex.h>
#include "stereo.h"

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	const int *dims;
	int ndims;
	uint16 rows, cols, dmax;
	uint8 disparities;
	IMAGEP leftDepth, rightDepth;
	IMAGEP cost = (IMAGEP)0;
	
	if (nrhs != 1)
	{
		mexErrMsgTxt("Usage: [leftDisp, rightDisp] = gdps(cost);");
	} 
	if (nlhs > 2)
	{
		mexErrMsgTxt("At most two output arguments allowed");
	}
	
	if (!(mxIsClass(prhs[0], "uint8")))
	{
		mexErrMsgTxt("Input array must be of type uint8");
	}
	
	ndims = mxGetNumberOfDimensions(prhs[0]);
	dims = mxGetDimensions(prhs[0]);
	if (ndims != 3)
	{
		mexErrMsgTxt("Input array must be three-dimensional");
	}

	rows = (uint16)dims[0];
	cols = (uint16)dims[1];
	disparities = (uint8)dims[2];
	dmax = disparities - 1;

	cost = blockFromBuffer(rows, cols, disparities, 1, t, f, (uint8 *)mxGetData(prhs[0]));
	
	plhs[0] = mxCreateNumericMatrix(rows, cols, mxUINT8_CLASS, mxREAL);
	plhs[1] = mxCreateNumericMatrix(rows, cols, mxUINT8_CLASS, mxREAL);

	leftDepth = imageFromBuffer(rows, cols, 1, 1, t, f, (uint8 *)mxGetData(plhs[0]));
	rightDepth = imageFromBuffer(rows, cols, 1, 1, t, f, (uint8 *)mxGetData(plhs[1]));

	dpStereo(cost, dmax, leftDepth, rightDepth);
}