#include "image.h"
#include <malloc.h>

/* Also initializes the new image with zeros */
IMAGEP imageFromBuffer(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign, uint8* buffer)
{
	IMAGEP ip;

	if (integer)
	{
		if (bytes != 1 && bytes != 2 && bytes != 4)
		{
			tell(t, "newImage: %d byte integer images not supported", bytes);
		}
	}
	else
	{
		if (bytes != 4 && bytes != 8)
		{
			tell(t, "newImage: %d byte real images not supported", bytes);
		}
		if (!sign)
		{
			tell(t, "newImage: unsigned real images not supported");
		}
	}
	
	ip = (IMAGEP)malloc(sizeof(IMAGE));

	ip->rows = rows;
	ip->cols = cols;
	ip->bands = bands;
	ip->bytes = bytes;
	ip->integer = integer;
	ip->sign = sign;

	ip->pixelBytes = bands * bytes;
	ip->rowBytes = ip->pixelBytes * ip->cols;
	ip->imageBytes = ip->rowBytes * ip->rows;

	if (buffer != (uint8 *)0)
	{
		ip->data = buffer;
	}
	else
	{
		ip->data = (uint8 *)calloc(ip->imageBytes, 1);
	}

	return ip;
}

IMAGEP newImage(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign)
{
	return imageFromBuffer(rows, cols, bands, bytes, integer, sign, (uint8 *)0);
}

void freeImagePixelBuffer(IMAGEP ip)
{
	if (ip != 0)
	{
		if (ip->data != 0)
		{
			free(ip->data);
			ip->data = (uint8 *)0;
		}
	}
}

void freeImage(IMAGEP *ip)
{
	if ((*ip) != 0)
	{
		freeImagePixelBuffer(*ip);
		free(*ip);
		*ip = 0;
	}
}

/* Also initializes the new block with zeros */
IMAGEP blockFromBuffer(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign, uint8* buffer)
{
	IMAGEP ip = imageFromBuffer(rows, cols, bands, bytes, integer, sign, buffer);

	ip->pixelBytes = bytes;
	ip->rowBytes = ip->pixelBytes * ip->cols;
	ip->bandBytes = ip->rowBytes * ip->rows;
	ip->imageBytes = ip->bandBytes * ip->bands;

	return ip;
}

IMAGEP newBlock(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign)
{
	return newImage(rows, cols, bands, bytes, integer, sign);
}

void freeBlockPixelBuffer(IMAGEP ip)
{
	freeImagePixelBuffer(ip);
}


void freeBlock(IMAGEP *ip)
{
	freeImage(ip);
}