#ifndef IMAGE_H
#define IMAGE_H

/* A brutally simple image data structure that also handles color images, signed/unsigned, real/integer pixels */

#include "tell.h"

typedef struct
{
	uint16 rows;
	uint16 cols;
	uint8 bands;
	uint8 bytes;
	logical integer;
	logical sign;
	uint8 *data;

	uint8 pixelBytes;
	uint32 rowBytes;
	uint32 bandBytes;
	uint32 imageBytes;
} IMAGE;

typedef IMAGE *IMAGEP;

#define PIXEL_ADDRESS(ip, r, c) ((int32)(((r) * (ip)->cols + (c)) * ((ip)->pixelBytes)))
#define PIXEL_POINTER(ip, r, c) ((ip)->data + PIXEL_ADDRESS(ip, r, c))
#define ROW_POINTER(ip, r) PIXEL_POINTER(ip, r, 0)

// Same, but from arrays coming from Matlab or Fortran, stored in column-major order
#define FORTRAN_PIXEL_ADDRESS(ip, r, c) ((int32)(((c) * (ip)->rows + (r)) * ((ip)->pixelBytes)))
#define FORTRAN_PIXEL_POINTER(ip, r, c) ((ip)->data + FORTRAN_PIXEL_ADDRESS(ip, r, c))
#define FORTRAN_ROW_POINTER(ip, r) FORTRAN_PIXEL_POINTER(ip, r, 0)

#define NEXT_PIXEL_POINTER(ip, pp) ((pp) + (ip)->pixelBytes)
#define INCREMENT_PIXEL_POINTER(ip, pp) (pp) += (ip)->pixelBytes

#define NEXT_ROW_POINTER(ip, rp) ((rp) + (ip)->rowBytes)
#define INCREMENT_ROW_POINTER(ip, rp) (rp) += (ip)->rowBytes

#define IS_IN_IMAGE_BOUNDS(ip, r, c, b) (0 <= (r) && (r) < ip->rows && 0 <= (c) && (c) < ip->cols && 0 <= (b) && (b) < ip->bands)

#define FORTRAN_BLOCK_ADDRESS(ip, r, c, b) \
	((int32)(((r) + (ip)->rows * ((c) + (b) * (ip)->cols)) * ((ip)->pixelBytes)))
#define FORTRAN_BLOCK_POINTER(ip, r, c, b) ((ip)->data + FORTRAN_BLOCK_ADDRESS(ip, r, c, b))

extern IMAGEP newImage(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign);
extern void freeImagePixelBuffer(IMAGEP ip);
extern void freeImage(IMAGEP *ip);
extern IMAGEP imageFromBuffer(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign, uint8 *buffer);

extern IMAGEP newBlock(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign);
extern void freeBlockPixelBuffer(IMAGEP ip);
extern void freeBlock(IMAGEP *ip);
extern IMAGEP blockFromBuffer(uint16 rows, uint16 cols, uint8 bands, uint8 bytes, logical integer, logical sign, uint8* buffer);

#endif // IMAGE_H