% Takes two census transforms and a vector with minimum and maximum
% disparity, and computes disparities left to right
function [d, Hout] = lrmatch(cl, cr, dbounds)

if nargout >= 2
    storeH = 1;
else
    storeH = 0;
end

rows = size(cl, 1);
cols = size(cl, 2);
neighbors = size(cl, 3);

disparity = min(dbounds):max(dbounds);
ndisp = length(disparity);

if storeH
    Hout = NaN + zeros(rows, cols, ndisp);
end

L = zeros(cols, neighbors, ndisp);
R = L;
d = zeros(rows, cols);
for r = 1:rows
    ll = squeeze(cl(r, :, :));
    rr = squeeze(cr(r, :, :));
    for k = 1:ndisp
        L(:, :, k) = ll;
        if disparity(k) >= 0
            R((1+disparity(k)):cols, :, k) = rr(1:(cols-disparity(k)), :);
        else
            R(1:(cols+disparity(k)), :, k) = rr((1-disparity(k):cols), :);
        end
    end
    H = squeeze(sum(L ~= R, 2));
    [val, pos] = min(H');
    d(r, :) = disparity(pos);
    
    if storeH
        Hout(r, :, :) = H;
    end
end